from flask import Flask, render_template, flash, \
            request, session, \
            redirect, url_for, logging, send_file

from flask_wtf import FlaskForm
from wtforms import StringField, TextAreaField, BooleanField, PasswordField, SelectField
from wtforms import validators
from wtforms import widgets, Form as _Form
from wtforms.validators import InputRequired

from wtforms.fields.html5 import DateField
from wtforms.fields.html5 import DateTimeField

import time
from datetime import datetime, timedelta

from flask_sqlalchemy import SQLAlchemy
from sqlalchemy.sql import text as SQLQuery

from flask_login import login_required

from main import app, db, mail, setting

import datamodel

import pandas as pd
from io import BytesIO

import ntpath

from models import WebLog



def chkNull(val):
    if val == None:
        return ''
    else:
        return val


class WebLogForm(FlaskForm):
    stDate = DateField("Start Date", format="'%Y-%m-%d'")
    enDate = DateField("End Date", format='%Y-%m-%d')
    action_by = StringField("Action by")
    action_taken = SelectField('Action Taken',
                                  choices=[
                                          ('', ''), \
                                          ('CREATE', 'CREATE'), \
                                          ('MODIFY', 'MODIFY'), \
                                          ('DELETE', 'DELETE'), \
                                          ('UPLOAD', 'UPLOAD') \
                                        ] \
                              )
    output = SelectField(u'Output Report To', choices=[('Web Browser', 'Web Browser'),('Excel', 'Excel')])
    

    def validate_on_submit(self):
        result = super(SmockForm, self).validate()
        if self.stDate.data is not None and self.enDate.data is not None:
            if (self.stDate.data > self.enDate.data):
                return False
            else:
                return True


@app.route('/weblogfilter', methods=['GET', 'POST'])
@login_required
def weblogfilter(sql=None, menu='dea'):
    if 'username' not in session:
        return redirect('/login')


    updlog = WebLog(timedate = datetime.now(),
                    action_taken = 'GET',
                    action_page = str(request.url).rsplit('/', 1)[1],
                    upd_by = session['userid'])
    db.session.add(updlog)
    db.session.commit()


    form = WebLogForm(request.form)
    table = datamodel.WebReportLog()

    cdata = ['Date/Time', 'ActionTaken', 'ActionBy', 'PreviousRecord', 'ActionResult']

    ddata = ()
    tables = []

    if request.method == 'POST':
        stDate = request.form['stDate']
        enDate = request.form['enDate']
        action_by = request.form['action_by']
        action_taken = request.form['action_taken']

        wherestDate = ''
        whereenDate = ''
        whereaction_by = ''
        whereaction_taken = ''

        strStDate = stDate
        strEnDate = enDate
        
        if stDate is not None and stDate != '':
            wherestDate = " AND timedate >= '" + str(stDate) + "' "
        if enDate is not None and enDate != '':
            tempStr = datetime.strptime(enDate, '%Y-%m-%d') + timedelta(days=1)
            tempStr1 = datetime.strftime(tempStr, '%Y-%m-%d')
            whereenDate = " and timedate <= '" + str(tempStr1) + "' "

        if wherestDate == '' and whereenDate != '':
            wherestDate = " AND timedate >= '" + str(enDate) + "' "
        if wherestDate != '' and whereenDate == '':
            tempStr = datetime.strptime(stDate, '%Y-%m-%d') + timedelta(days=1)
            tempStr1 = datetime.strftime(tempStr, '%Y-%m-%d')
            whereenDate = " AND timedate <= '" + str(tempStr1) + "' "

        if action_by is not None and action_by != '':
            whereaction_by = " AND upd_by = '" + str(action_by) + "' "
        if action_taken is not None and action_taken != '':
            whereaction_taken = " AND action_taken = '" + str(action_taken) + "' "


        if setting.dbtype.upper() == 'MSSQL':
            mssqllimit = ' TOP 2000 '
            mysqllimit = ''
        else:
            mssqllimit = ''
            mysqllimit = ' LIMIT 2000 '


        subSQL = "SELECT " + mssqllimit + " " + \
                 "timedate, action_taken, upd_by, " + \
                 "previous_record, action_sql " + \
                 "FROM weblog " + \
                 "WHERE 1=1 " + \
                 "AND action_taken <> 'GET' " + \
                 wherestDate + \
                 whereenDate + \
                 whereaction_by + \
                 whereaction_taken

        subSQL = subSQL + " ORDER by timedate DESC "
        subSQL = subSQL + mysqllimit

##        sql = SQLQuery(subSQL)
##        ddata = db.session.execute(sql).fetchall()

        pdSQL = pd.read_sql_query(subSQL, db.get_engine(app))
        df = pd.DataFrame(pdSQL)

        df.columns = cdata


        if request.form['output'].upper() == 'EXCEL':
            output = BytesIO()
            writer = pd.ExcelWriter(output, engine='xlsxwriter')
            

        df.reset_index(drop=True, inplace=True)
        df.index = df.index + 1

        pd.set_option('display.max_colwidth', -1)

        df.style.set_properties(**{
                                    'text-align': 'left',
                                    'white-space': 'pre-wrap',
                                    'font-size': '20pt',
                                })
        df.style.set_properties(**{'font-size':'6pt'})
        df.style.applymap('color:red')
        df.style.set_properties(**{'background-color': 'black',
                           'color': 'lawngreen',
                           'border-color': 'white'})


        # use this way column will become text
        df['Date/Time'] = df['Date/Time'].dt.strftime('%Y-%m-%d %H:%M:%S')
        
        for index in df.index:
            df.loc[index, 'Date/Time'] = chkNull(df.loc[index, 'Date/Time'])
            df.loc[index, 'ActionTaken'] = chkNull(df.loc[index, 'ActionTaken'])
            #df.loc[index, 'Page'] = chkNull(df.loc[index, 'Page'])
            df.loc[index, 'ActionBy'] = chkNull(df.loc[index, 'ActionBy'])
            df.loc[index, 'PreviousRecord'] = chkNull(df.loc[index, 'PreviousRecord']).rstrip("<br>").replace('<br>', '\n')

            if df.loc[index, 'ActionResult'] is not None:
                if df.loc[index, 'ActionTaken'] == 'CREATE':
                    str1 = str(df.loc[index, 'ActionResult']).split("VALUES(", 1)[0]
                    str1 = str1.split("(", 1)[1]
                    str2 = str(df.loc[index, 'ActionResult']).split("VALUES(", 1)[1]
                    str1 = str1.replace(" ", "")
                    str2 = str2.replace(" ", "")
                    arr1 = []
                    strall = ""

                    for str1x in str1[:-1].split(","):
                        arr1.append(str(str1x))

                    n = 0    
                    for str2x in str2[:-1].split(","):
                        strall = strall + arr1[n] + " = " + str(str2x) + "\n"
                        n = n + 1

                    strall = strall.replace("'", "")
                    strall = strall.rstrip("\n")

                    df.loc[index, 'ActionResult'] = strall
                    
                elif df.loc[index, 'ActionTaken'] == 'MODIFY':
                    df.loc[index, 'ActionResult'] = str(df.loc[index, 'ActionResult']).split("SET", 1)[1]
                    df.loc[index, 'ActionResult'] = str(df.loc[index, 'ActionResult']).split("WHERE", 1)[0]
                    df.loc[index, 'ActionResult'] = str(df.loc[index, 'ActionResult']).replace("', ", "\n")
                    df.loc[index, 'ActionResult'] = str(df.loc[index, 'ActionResult']).replace("'", "").strip()
                elif df.loc[index, 'ActionTaken'] == 'DELETE':
                    df.loc[index, 'ActionResult'] = str(df.loc[index, 'ActionResult']).split("WHERE", 1)[1]
                    df.loc[index, 'ActionResult'] = "DELETE " + str(df.loc[index, 'ActionResult']).replace("'", "")
                elif df.loc[index, 'ActionTaken'] == 'EXPORT':
                    df.loc[index, 'ActionResult'] = ""
                else:
                    df.loc[index, 'ActionResult'] = chkNull(df.loc[index, 'ActionResult']).replace("<br>", "\n")
            df.loc[index, 'ActionResult'] = chkNull(df.loc[index, 'ActionResult'])

            if df.loc[index, 'ActionTaken'] == 'UPLOAD':
                df.loc[index, 'ActionResult'] = "Upload Success"

##            if df.loc[index, 'UploadFile'] is not None:
##                df.loc[index, 'UploadFile'] = ntpath.basename(df.loc[index, 'UploadFile'])
##            df.loc[index, 'UploadFile'] = chkNull(df.loc[index, 'UploadFile'])


        
        if request.form['output'].upper() == 'EXCEL':
            df.to_excel(writer, merge_cells=False)

            workbook  = writer.book
            worksheet = writer.sheets['Sheet1']
            valign_format = workbook.add_format({'valign': 'top'})
            wrap_format = workbook.add_format({'valign': 'vcenter', 'text_wrap': True})

##            # setting the format but not setting the column width
            worksheet.set_column('B:H', None, valign_format)
##            worksheet.set_column('F:G', None, wrap_format)

##            # let column F,G wrap text format
##            # to prevent column too long
            worksheet.set_column('F:G', 35, wrap_format)

            writer.close()
            
            output.seek(0)


            updlog = WebLog(timedate = datetime.now(),
                            action_taken = 'EXPORT',
                            action_page = str(request.url).rsplit('/', 1)[1],
                            upd_by = session['userid'],
                            action_sql = subSQL)
            db.session.add(updlog)
            db.session.commit()
            
        
            return send_file(output, \
                             attachment_filename="WebLog.xlsx", \
                             as_attachment=True)

##        # replace all <br> to \n
##        # save <br> in database
##        # because \n may not able view in database GUI tools
##        # use \n because can use for excel also
##        tables = [df.to_html().replace("\\n","<br>")]

        
##        # escape set to false can show html element <>& correctly in to_html
##        # otherwise it will print <>& sign in html page
##        # for example if escape no set to false
##        # &nbsp; will no show space but print &nbsp;
##        # and <br> no show new line but print <br>
        #tables = [df.to_html(escape=False)]


##        # since it may not only use for html in future
##        # better do standard to support both html and excel
        col_nowrap = lambda x: "<span style='white-space: nowrap;'>" + x + "</span>"
        tables = [df.to_html(classes='table table-striped table-bordered table-hover table-condensed', \
                             formatters={'Date/Time':col_nowrap}, \
                             escape=False) \
                  .replace("\\n","<br>")]
        

    return render_template('weblog.html', form=form, data=ddata, table=table, mycolumns=cdata, tables=tables)
