from flask import Flask, render_template, request, redirect, url_for, \
     flash, session, logging, send_from_directory
from flask_sqlalchemy import SQLAlchemy
from sqlalchemy.sql import text as SQLQuery
from wtforms import Form, StringField, TextAreaField, PasswordField, validators
from wtforms.validators import DataRequired

from flask_ckeditor import CKEditor, CKEditorField, upload_fail, upload_success

from main import app, session, db, is_logged_in

from flask_login import login_required


# Article Form Class
class ArticleForm(Form):
    title = StringField('Title', [validators.Length(min=1, max=200)])
    body = TextAreaField('Body', [validators.Length(min=1)])
    

# Articles
@app.route('/articles')
def articles():
    # Get articles
    querystring = "SELECT * FROM articles " + \
                  "ORDER BY id DESC "
    sql = SQLQuery(querystring)
    articles = db.session.execute(sql).fetchall()

    if len(articles) > 0:
        return render_template('articles.html', articles=articles)
    else:
        msg = 'No Articles Found'
        return render_template('articles.html', msg=msg)
    

# Single Article
@app.route('/article/<string:id>/')
def article(id):
    # Get article
    querystring = "SELECT * FROM articles " + \
                  "WHERE id = '" + str(id) + "' "
    sql = SQLQuery(querystring)
    article = db.session.execute(sql).fetchone()
    
    return render_template('article.html', article=article)



# Add Article
@app.route('/add_article', methods=['GET', 'POST'])
@login_required
def add_article():
    form = ArticleForm(request.form)
    if request.method == 'POST' and form.validate():
        title = form.title.data
        body = form.body.data

        # Execute
        querystring = "INSERT INTO articles(title, body, author) " + \
                      "VALUES('" + title + "', '" + body + "', '" + session['username'] + "') "
        sql = SQLQuery(querystring)
        db.session.execute(sql)
        db.session.commit()

        flash('Article Created', 'success')

        return redirect(url_for('dashboard'))

    return render_template('add_article.html', form=form)



# Edit Article
@app.route('/edit_article/<string:id>', methods=['GET', 'POST'])
@login_required
def edit_article(id):
    # Get article by id
    querystring = "SELECT * FROM articles " + \
                  "WHERE id = '" + str(id) + "' "
    sql = SQLQuery(querystring)
    article = db.session.execute(sql).fetchone()

    # Get form
    form = ArticleForm(request.form)

    # Populate article form fields
    form.title.data = article['title']
    form.body.data = article['body']
    
    if request.method == 'POST' and form.validate():
        title = request.form['title']
        body = request.form['body']

        # Execute
        querystring = "UPDATE articles SET " + \
                      "title ='" + title + "', " + \
                      "body = '" + body + "' " + \
                      "WHERE id = '" + str(id) + "' "
        #flash(querystring)
        sql = SQLQuery(querystring)
        db.session.execute(sql)
        db.session.commit()

        flash('Article Updated', 'success')

        return redirect(url_for('dashboard'))

    return render_template('edit_article.html', form=form)


# Delete article
@app.route("/delete_article/<string:id>", methods=['POST'])
@login_required
def delete_article(id):
    # Execute
    querystring = "DELETE FROM articles " + \
                  "WHERE id = '" + str(id) + "' "
    #flash(querystring)
    sql = SQLQuery(querystring)
    db.session.execute(sql)
    db.session.commit()

    flash('Article Deleted', 'success')

    return redirect(url_for('dashboard'))
