from flask import Flask, render_template, request, redirect, url_for, \
     flash, session, logging, send_from_directory
from flask_sqlalchemy import SQLAlchemy
from sqlalchemy.sql import text as SQLQuery

from main import app, session, db, is_logged_in, setting, login_manager

from datetime import datetime


import uuid

from wtforms import Form, StringField, TextAreaField, PasswordField, validators

from models import *

from flask_login import login_required, login_user, logout_user

from dateutil.relativedelta import *


# Login Form Class
class LoginForm(Form):
    userid = StringField('UserID', validators=[validators.DataRequired()])
    password = PasswordField('Password', validators=[validators.DataRequired()])


def updLoginStatus(userid, fail_login=0, last_login=0, session_token=None, password_last_change=None):

    if setting.dbtype.upper() == 'MSSQL':
        tablename = '[user]'
        mysysdate = 'GETDATE() '
    else:
        tablename = 'user'
        mysysdate = 'SYSDATE() '
    
    strSQL = "UPDATE " + tablename +  " SET " + \
             "fail_login = '" + str(fail_login) + "' "

    #print(last_login)

    if last_login == 1:
        strSQL = strSQL + \
                 ", last_login = " + mysysdate + " "

    if session_token is not None:
        strSQL = strSQL + \
                 ", session_token = '" + session_token + "' "

    if password_last_change is not None:
        strSQL = strSQL + \
                 ", password_last_change = '" + password_last_change + "' "

    strSQL = strSQL + \
             "WHERE userid = '" + userid + "' "
    
    #flash(strSQL)

    sql = SQLQuery(strSQL)
    db.session.execute(sql)
    db.session.commit()
    
    



# User login
@app.route('/login', methods=['GET', 'POST'])
def login():
    form = LoginForm(request.form)
    if request.method == 'POST' and form.validate():
        loginuser = User.query.filter_by(userid=form.userid.data).first()

        if loginuser is not None:
            password = loginuser.password
            last_login = loginuser.last_login
            fail_login = loginuser.fail_login
            check_password_expired = loginuser.check_password_expired
            password_last_change = loginuser.password_last_change
            
            if password is None:
                password = '123456'

            if last_login is None:
                last_login = datetime.now()
                updLoginStatus(userid = loginuser.userid,
                               last_login = 1)
            if fail_login is None:
                fail_login = 0

            if check_password_expired is None:
                check_password_expired = True
            else:
                if str(check_password_expired).upper().find('FALSE') >= 0:
                    check_password_expired = False
                else:
                    check_password_expired = True

            if password_last_change is None:
                password_last_change = datetime.now()

                updLoginStatus(userid = loginuser.userid,
                               password_last_change = datetime.now().strftime("%Y-%m-%d %H:%M:%S"))

            loginlockdiff = datetime.now() - last_login
            lastpassworddiff = datetime.now() - password_last_change
                

            # login condition
            if check_password_expired == True and lastpassworddiff.days > 90:
                error = "This username's password is expired, please reset your password"

                updlog = WebLog(timedate = datetime.now(),
                                action_taken = 'LOGIN',
                                action_page = str(request.url).rsplit('/', 1)[1],
                                upd_by = form.userid.data,
                                action_sql = 'Account suspended for password expired')
                db.session.add(updlog)
                db.session.commit()
                
            elif form.password.data == password:
                #print(fail_login)
                #print(loginlockdiff.total_seconds())

                if fail_login >= 3 and loginlockdiff.total_seconds() < 1800:
                    # lock for 30 minutes

                    error = 'The account has been locked. ' + \
                            'Please try again in 30 minutes. '

                    updlog = WebLog(timedate = datetime.now(),
                                    action_taken = 'LOGIN',
                                    action_page = str(request.url).rsplit('/', 1)[1],
                                    upd_by = form.userid.data,
                                    action_sql = 'Login fail, password incorrect >= 3, account suspended 30 minutes')
                    db.session.add(updlog)
                    db.session.commit()
                    
                else:
                    # Passed

                    # check duplicate login
                    substrSQL = "SELECT TOP 1 timedate, action_taken, " + \
                             "action_page, upd_by, action_sql FROM weblog " + \
                             "WHERE upd_by = '" + str(form.userid.data) + "' " + \
                             "AND (action_sql NOT LIKE '%LOGIN FAIL%' " + \
                             "      OR action_sql IS NULL) " + \
                             "ORDER BY timedate DESC "
                    #print(substrSQL)
                    subsql = SQLQuery(substrSQL)
                    myrow = db.session.execute(subsql).fetchone()

                    myLoginStatus = True

                    if myrow is not None:
                        print(myrow['timedate'] + relativedelta(seconds=900))
                        print(myrow['action_taken'])
                        if myrow['action_taken'] != 'LOGOUT':
                            if myrow['timedate'] + relativedelta(seconds=900) >= datetime.now():
                                myLoginStatus = False
                            

                    if myLoginStatus != True:
                        # lock for duplicate login

                        error = 'The account has been logged in at another site. ' + \
                                'Please try again later. '

                        updlog = WebLog(timedate = datetime.now(),
                                        action_taken = 'LOGIN',
                                        action_page = str(request.url).rsplit('/', 1)[1],
                                        upd_by = form.userid.data,
                                        action_sql = 'Login fail, duplicate login')
                        db.session.add(updlog)
                        db.session.commit()
                        
                    else:
                        session['logged_in'] = True
                        session['userid'] = loginuser.userid
                        session['username'] = loginuser.username
                        session['accessright'] = loginuser.accessright

                        session['token'] = loginuser.userid + '_' + datetime.now().strftime('%Y%m%d_%H%M%S') + '_' + str(uuid.uuid4())

                        updLoginStatus(userid = loginuser.userid,
                                       fail_login = 0,
                                       last_login = 1,
                                       session_token = str(session['token']))

                        login_user(loginuser)


                        
                        updlog = WebLog(timedate = datetime.now(),
                                        action_taken = 'LOGIN',
                                        action_page = str(request.url).rsplit('/', 1)[1],
                                        upd_by = form.userid.data,
                                        action_sql = 'Login success')
                        db.session.add(updlog)
                        db.session.commit()
                    
                    

                        #flash('You are now logged in', 'success')
                        return redirect(url_for('index'))
            
            else:
                error = 'Password incorrect'
                
                if fail_login >= 2:
                    if loginlockdiff.total_seconds() < 1800:
##                        # fail more than 3 times and lock 30 minutes
##                        t = 1800 - loginlockdiff.total_seconds()
##                        mins, secs = divmod(t, 60)
##                        timeformat = '%02d:%02d'%(mins, secs,)
##
##                        error = 'This username login failed more than 3 times, ' + \
##                                'please try again after ' + str(timeformat)
                        error = 'The account has been locked. ' + \
                                'Please try again in 30 minutes. '

                        if fail_login < 3:
                            fail_login = fail_login + 1
                            updLoginStatus(userid = loginuser.userid,
                                           fail_login = fail_login,
                                           last_login = 1)

                        updlog = WebLog(timedate = datetime.now(),
                                        action_taken = 'LOGIN',
                                        action_page = str(request.url).rsplit('/', 1)[1],
                                        upd_by = form.userid.data,
                                        action_sql = 'Login fail, password incorrect >= 3, account suspended 30 minutes')
                        db.session.add(updlog)
                        db.session.commit()
                    else:
                        updLoginStatus(userid = loginuser.userid,
                                       fail_login = 1,
                                       last_login = 0)

                        updlog = WebLog(timedate = datetime.now(),
                                        action_taken = 'LOGIN',
                                        action_page = str(request.url).rsplit('/', 1)[1],
                                        upd_by = form.userid.data,
                                        action_sql = 'Login fail, password incorrect (' + str(fail_login) + ")")
                        db.session.add(updlog)
                        db.session.commit()
                        
                else:
                    fail_login = fail_login + 1
                    updLoginStatus(userid = loginuser.userid,
                                   fail_login = fail_login,
                                   last_login = 1)

                    
                    updlog = WebLog(timedate = datetime.now(),
                                    action_taken = 'LOGIN',
                                    action_page = str(request.url).rsplit('/', 1)[1],
                                    upd_by = form.userid.data,
                                    action_sql = 'Login fail, password incorrect (' + str(fail_login) + ")")
                    db.session.add(updlog)
                    db.session.commit()
                
                
            return render_template('login.html', form=form, error=error)
        
        else:
            error = 'Username not found'
            return render_template('login.html', form=form, error=error)
            

    return render_template('login.html', form=form)


# Logout
@app.route('/logout')
@login_required
def logout():

    updlog = WebLog(timedate = datetime.now(),
                    action_taken = 'LOGOUT',
                    action_page = str(request.url).rsplit('/', 1)[1],
                    upd_by = session['userid'],
                    action_sql = 'Logout Success')
    db.session.add(updlog)
    db.session.commit()
                
    login_manager.login_message = None
    logout_user()
    session.clear()
    #flash('You are now logged out', 'success')
    return render_template('logout.html')
