#-*- coding: utf-8 -*-

from flask import Flask, render_template, flash, \
             redirect, url_for, session, request, logging, \
             send_from_directory


from flask_sqlalchemy import SQLAlchemy
from sqlalchemy.sql import text as SQLQuery

from wtforms import Form, StringField, TextAreaField, PasswordField, validators
#from passlib.hash import sha256_crypt
from functools import wraps

from datetime import datetime, timedelta

from werkzeug.utils import secure_filename
from flask_mail import Message, Mail
import os, sys
import xlrd
import uuid

from setting import *

import decimal

import pandas as pd


from flask_login import LoginManager, login_required


basedir = os.path.abspath(os.path.dirname(__file__))

UPLOAD_FOLDER = os.path.join(basedir, 'upload')
ALLOWED_EXTENSIONS = set(['xls', 'xlsx'])

app = Flask(__name__)
app.config['SECRET_KEY'] = "ida6422690"
app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER

#####################################
#email
app.config['MAIL_SERVER'] = setting.smtpserver
app.config['MAIL_PORT'] = setting.smtpport
app.config['MAIL_USERNAME'] = setting.smtpuserid
app.config['MAIL_PASSWORD'] = setting.smtppassword
app.config['MAIL_USE_TLS'] = False

#print('smtppassword: ' + str(setting.smtppassword.strip()) + '#')
if setting.smtppassword.strip() == "":
    app.config['MAIL_USE_SSL'] = False
else:
    app.config['MAIL_USE_SSL'] = True

app.config['MAIL_DEFAULT_SENDER'] = (setting.smtpsendername, setting.smtpsender)
#####################################
mail = Mail(app)

# Config Database
if setting.dbtype.upper() == 'MSSQL':
    dbdriver = 'mssql+pymssql'
else:
    dbdriver = 'mysql'

conn_str = dbdriver + '://' + \
           str(setting.dbuserid) + \
           ':' + str(setting.dbpassword) + \
           '@' + str(setting.dbserver) + ':' + str(int(setting.dbport)) + \
           '/' + str(setting.dbschema)
app.config['SQLALCHEMY_DATABASE_URI'] = conn_str
app.config['SQLALCHEMY_BINDS'] = {
                                'localdb': 'mysql://ida:ida6422690@localhost:3306/idaesd'
                                }

# init MySQL
db = SQLAlchemy(app)


# init Login Manager
login_manager = LoginManager()
login_manager.init_app(app)
login_manager.login_view = 'login'
login_manager.login_message = None


@app.before_request
def make_session_permanent():
    session.permanent = True
##    app.permanent_session_lifetime = timedelta(minutes=15)
    app.permanent_session_lifetime = timedelta(minutes=150)


@app.context_processor
def get_app_setting():
    return dict(setting=mysetting())



# Reporting ##################################
@app.template_filter('to_decimal')
def to_decimal_filter(f):
    return decimal.Decimal(f)

@app.template_filter('to_decimal_normalize')
def to_decimal_normalize_filter(f):
    x = decimal.Decimal(f)
    return x.normalize()

@app.template_filter('to_decimal_normalize_x')
def to_decimal_normalize_x_filter(f):
    try:
        x = decimal.Decimal(f)
        y = x.normalize()
        return str('{:.4E}'.format(float(y)))
    except:
        return str(f)

@app.template_filter('chknull')
def chkNull(val):
    if val == None:
        return ''
    else:
        return val
##############################################




# Check if user logged in
def is_logged_in(f):
    @wraps(f)
    def wrap(*args, **kwargs):
        if 'logged_in' in session:
            return f(*args, **kwargs)
        else:
            #flash('Unauthorized, Please login', 'danger')
            return redirect(url_for('login'))
    return wrap



@app.route("/tables")
def show_tables():
##    data = pd.read_excel('C:\Users\User\Desktop\dummy_data.xlsx')
##    data.set_index(['Name'], inplace=True)
##    data.index.name=None
##    females = data.loc[data.Gender=='f']
##    males = data.loc[data.Gender=='m']
##    print(females)
##    return render_template('view.html',tables=[females.to_html(classes='female'), males.to_html(classes='male')],
##    titles = ['na', 'Female surfers', 'Male surfers'])

#####################################################################################
    data = pd.read_excel('C:\Users\User\Desktop\Report.xlsx')
    females = data.loc[data.SupID==83140]
    males = data.loc[data.SupID==40402]

    ### >>> begin multiple column
    header = ['EmpID', \
              'Name', \
              'CardNo', \
              'SupID', \
              'Department', \
              'Shift', \
              'Email', \
              'CostCenter']
    header = list(zip(['HEADER'] * 8, header))             
    #[('HEADER', 'A'), ('HEADER', 'B'), ('HEADER', 'C'), ('HEADER', 'D')]
    header = pd.MultiIndex.from_tuples(header)    
    ### >>> end multiple column

    df = pd.DataFrame(females, columns=header)
    df.reset_index(drop=True, inplace=True)
    df.index = df.index + 1

    df2 = pd.DataFrame(males, columns=['EmpID', \
                                        'Name', \
                                        'CardNo', \
                                        'SupID', \
                                        'Department', \
                                        'Shift', \
                                        'Email', \
                                        'CostCenter'])
    df2.reset_index(drop=True, inplace=True)
    df2.index = df2.index + 1

    pd.set_option('display.max_colwidth',100)
    
    return render_template('view.html',\
                           tables=[df.to_html(classes='female'), \
                                   df2.to_html(classes='male')],\
                           titles = ['na'])
#####################################################################################
##    data = pd.read_excel('C:\Users\User\Desktop\Report.xlsx')
##    #print(data.SupID.unique())
##
##    supID_List = data.SupID.unique()
##    #print(supID_List)
##    tblReport = []
##    x = 0
##    for supid in supID_List:
##        tblData = data.loc[data.SupID==supid]
##        df = pd.DataFrame(tblData, columns=['EmpID', \
##                                     'Name', \
##                                     'CardNo', \
##                                     'SupID', \
##                                     'Department', \
##                                     'Shift', \
##                                     'Email', \
##                                     'CostCenter'])
##    
##        df.reset_index(drop=True, inplace=True)
##        df.index = df.index + 1
##        tblReport.insert(x, tblData.to_html())
##        x = x + 1
##
##    pd.set_option('display.max_colwidth',100)
##    
####    return render_template('view.html',tables=[df.to_html(classes='female'), males.to_html(index=False, classes='male')],
####    titles = ['na', 'Female surfers', 'Male surfers'])
##    return render_template('view.html',\
##                           tables=tblReport,\
##                           titles = ['na'])




def allowed_file(filename):
    return '.' in filename and \
           filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS

@app.route('/uploademployee', methods=['GET', 'POST'])
@login_required
def upload_file():
    if 'username' not in session:
        return redirect('/login')

    querystring = "INSERT INTO weblog(timedate, action_taken, action_page, upd_by) " + \
                  "VALUES(GETDATE(), 'GET', 'uploademployee', '" + session['userid'] + "') "
    sql = SQLQuery(querystring)
    db.session.execute(sql)
    db.session.commit()
    
    if request.method == 'POST':
        # check if the post request has the file part
        if 'file' not in request.files:
            flash('No file part')
            return redirect(request.url)
        file = request.files['file']
        # if user does not select file, browser also
        # submit a empty part without filename
        if file.filename == '':
            flash('No selected file')
            return redirect(request.url)
        if file and allowed_file(file.filename):
            filename = secure_filename(file.filename)
            extension = filename.split('.')[1].lower()
            unique_filename = 'EMP_' + datetime.now().strftime('%Y%m%d_%H%M%S') + '_' + str(uuid.uuid4())
            filename = unique_filename + '.' + extension
            file.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))
            uploadstatus = filetoupload(filename)

            if uploadstatus == True:
                print("Upload status = true")
                return redirect(url_for('uploaded_file',
                                        filename=filename)) #to download on current pc
            else:
                print("Upload status is not true")

    return render_template('processfile.html')

@app.route('/uploads/<filename>')
def uploaded_file(filename):
    
##    send_from_directory(app.config['UPLOAD_FOLDER'],
##                               filename)
    return render_template('doneuploadfile.html')
##    return redirect(url_for('filetoupload'))

def getmysqlstr(fldval):
    try:
        if fldval is None:
            return 'Null, '
        else:
            #return "\'" + str(fldval).replace("\'", "\\'") + "\',
            #return "\'" + str(fldval).replace("@", "\@'") + "'\ "
            #return "\'" + fldval + "'\ "
            return "" + fldval + ""

    except Exception as e:
        #print 'Error getmysqlstr(): ' + str(e)
        return 'Null, '


def chkFloat(fldval):
    try:
        val = str(float(fldval))
    except:
        val = "null"

    return val


def chkNull(val):
    if val == None:
        return ''
    else:
        return val

def chkEmpDuplicate(empid):
    strSQL = "SELECT * FROM employee " + \
             "WHERE emp_id = '" + str(chkNull(empid)) + "' "
    sql = SQLQuery(strSQL)
    data = db.session.execute(sql).fetchall()

##    print(data)
##    print(len(data))

    if len(data) < 1:
        return -1
    else:
        return 1
        

def filetoupload(filename):
    global book
    global sheet

    #print("filetopuload: " + filename)
    # Open the workbook and define the worksheet
    #path_upload = "./Upload/" + str(filename)
    #path_upload =  app.config['UPLOAD_FOLDER'] + str(filename)
    path_upload = os.path.join(app.config['UPLOAD_FOLDER'], filename)
    #print "path_upload:::" + str(path_upload)
    book = xlrd.open_workbook(path_upload)
    #sheet = book.sheet_by_name("Sheet1")
    sheet = book.sheet_by_index(0)

    #print("Successfully open Excel file")

    txtAction = str(request.form['txtAction']).upper()

    uploadstatus = True

    try:
        # for weblog ######################################
        print("save to weblog start")
        values = {'timedate':'', 'action_taken':'', 'action_page':'', \
                  "upd_by":'', 'upload_file_path':''}
        
        values['timedate'] = datetime.now()
        values['action_taken'] = 'UPLOAD'
        values['action_page'] = str(request.url).rsplit('/', 1)[1]
        values['upd_by'] = session['userid']
        values['upload_file_path'] = path_upload
        
        strSQL = "INSERT INTO weblog(timedate, action_taken, action_page, upd_by, upload_file_path) " + \
                 "VALUES(:timedate, :action_taken, :action_page," + \
                        ":upd_by, :upload_file_path)"
        sql = SQLQuery(strSQL)
        db.session.execute(sql, values)
        db.session.commit()
        print("save to weblog done")
        ###################################################

        
        if txtAction.find('OVERWRITE') >= 0:
            strSQL =  " UPDATE employee " + \
                      " SET updstatus = 'UPDATING' " + \
                      " WHERE emp_id <> 'DEFAULT' " 
            sql = SQLQuery(strSQL)
            db.session.execute(sql)
            db.session.commit()
        

        
##        query = "INSERT INTO employee (emp_id, emp_name, " + \
##                "card_no, sup_id, department, shift, email, costcenter) " + \
##                "VALUES (%s, %s, %s, %s, %s, %s, %s, %s) "

        # Create a For loop to iterate through each row in the XLS file, starting at row 2 to skip the headers
        for r in range(1, sheet.nrows):
            empid = ''
            empname = ''
            cardno = ''
            supid = ''
            department = ''
            shift = ''
            email = ''
            costcenter = ''
            cellvalue = ''
            for c in range(0, sheet.ncols):
                mycolname =  str(sheet.cell(0,c).value).strip().upper()
##                print(mycolname)

                if sheet.cell(r,c).value is not None:
                    #cellvalue = str(sheet.cell(r,c).value)
                    dvalue = sheet.cell(r,c).value
                    if type(dvalue) == float:
                        cellvalue = '{:05.0f}'.format(dvalue)
                    else:
                        cellvalue = dvalue
                    
                    if mycolname.find('EMPLOYEEID') >= 0 \
                       or mycolname.find('EMPID') >= 0 \
                       or mycolname.find('EMP_ID') >= 0 \
                       or mycolname.find('EMPNO') >= 0:
                        
                        empid	    = cellvalue
                        
                    elif mycolname.find('NAME') >= 0:
                        empname	    = cellvalue
                    elif mycolname.find('CARD') >= 0 or mycolname.find('BADGE') >= 0:
                        cardno	    = cellvalue
                        
                    elif mycolname.find('SMOCKID') >= 0 \
                         or mycolname.find('SMOCK_ID') >= 0 \
                         or mycolname.find('SMOCKNO') >= 0 \
                         or mycolname.find('SMOCK_NO') >= 0:

                        empid       = cellvalue
                        
                    elif mycolname.find('SUP') >= 0 and mycolname.find('ID') >= 0:
                        supid       = cellvalue
                    elif mycolname.find('DEPT') >= 0 or mycolname.find('DEPARTMENT') >= 0:
                        department  = cellvalue
                    elif mycolname.find('SHIFT') >= 0:
                        shift	    = cellvalue
                    elif mycolname.find('EMAIL') >= 0:
                        email	    = cellvalue
                    elif mycolname.find('COST') >= 0 \
                         or mycolname.find('COSTCENTER') >= 0:
                        costcenter  = cellvalue

##            ## Assign values from each row
##            # for insert
##            values = "'" + str(chkNull(empid)) + "' " + \
##                     ", '" + str(chkNull(empname)).replace("'", "''") + "' " + \
##                     ", '" + str(chkNull(cardno)) + "' " + \
##                     ", '" + str(chkNull(supid)) + "' " + \
##                     ", '" + str(chkNull(department)).replace("'", "''") + "' " + \
##                     ", '" + str(chkNull(shift)) + "' " + \
##                     ", '" + str(chkNull(email)) + "' " + \
##                     ", '" + str(chkNull(costcenter)) + "' "
##            # for update
##            query2 = "  emp_id      = '" + str(chkNull(empid)) + "' " \
##                     ", emp_name    = '" + str(chkNull(empname)).replace("'", "''") + "' " \
##                     ", card_no     = '" + str(chkNull(cardno)) + "' " \
##                     ", sup_id      = '" + str(chkNull(supid)) + "' " \
##                     ", department  = '" + str(chkNull(department)).replace("'", "''") + "' " \
##                     ", shift       = '" + str(chkNull(shift)) + "' " \
##                     ", email       = '" + str(chkNull(email)) + "' " \
##                     ", costcenter  = '" + str(chkNull(costcenter)) + "' " \
##                     ", updstatus  = 'UPDATED' "


            ## Assign values from each cell for a row
            values = {'empid':'', 'empname':'', 'cardno':'', "supid":'', \
                      "department":'', 'shift':'', 'email':'', 'costcenter':''}
            values['empid'] = empid
            values['empname'] = empname
            values['cardno'] = cardno
            values['supid'] = supid
            values['department'] = department
            values['shift'] = shift
            values['email'] = email
            values['costcenter'] = costcenter

            #print("")
            #print("*********************")
            #print(str(values))
            #print("")

            if chkEmpDuplicate(empid) == -1:
                #not found in database
                query = "INSERT INTO employee " + \
                        "       (emp_id, emp_name, " + \
                        "       card_no, sup_id, department," + \
                        "       shift, email, costcenter, updstatus) " + \
                        "VALUES(:empid, :empname, :cardno, :supid, " + \
                        ":department, :shift, :email, :costcenter, 'UPDATED')"
            else:
                #already exist in database
                query = "UPDATE employee SET " + \
                        "emp_id = :empid " + \
                        ", emp_name = :empname " + \
                        ", card_no = :cardno " + \
                        ", sup_id = :supid " + \
                        ", department = :department " + \
                        ", shift = :shift " + \
                        ", email = :email " + \
                        ", costcenter = :costcenter " + \
                        ", updstatus = 'UPDATED' " + \
                        "WHERE emp_id = :empid "

            # Execute sql Query
            sql = SQLQuery(query)
            db.session.execute(sql, values)
            db.session.commit()


        if txtAction.find('OVERWRITE') >= 0:
            strSQL = " DELETE FROM employee " + \
                     " WHERE emp_id <> 'DEFAULT' " + \
                     " AND updstatus <> 'UPDATED' "
            sql = SQLQuery(strSQL)
            db.session.execute(sql)
            db.session.commit()
        
            
    except Exception as e:
        #print(str(e))
        flash("Upload Error", 'danger')
        #flash(query)
        flash(str(e))
        uploadstatus = False

    return uploadstatus



# Index
@app.route('/')
@login_required
def index():
    querystring = "INSERT INTO weblog(timedate, action_taken, action_page, upd_by) " + \
                  "VALUES(GETDATE(), 'GET', 'index', '" + session['userid'] + "') "
    sql = SQLQuery(querystring)
    db.session.execute(sql)
    db.session.commit()
    
    return render_template('home.html')

# About
@app.route('/about')
@login_required
def about():
    querystring = "INSERT INTO weblog(timedate, action_taken, action_page, upd_by) " + \
                  "VALUES(GETDATE(), 'GET', 'about', '" + session['userid'] + "') "
    sql = SQLQuery(querystring)
    db.session.execute(sql)
    db.session.commit()
    return render_template('about.html')


# Dashboard (for article traffic light testing)
@app.route('/dashboard')
@login_required
def dashboard():
    # Get articles
    querystring = "SELECT * FROM articles"
    sql = SQLQuery(querystring)
    articles = db.session.execute(sql).fetchall()

    if len(articles) > 0:
        return render_template('dashboard.html', articles=articles)
    else:
        msg = 'No Articles Found'
        return render_template('dashboard.html', msg=msg)



from login import *
#from register import *     # for traffic light
#from article import *      # for blog testing
from tblshow import *
from chgpassword import *
from smockfilter import *
from pwreset import *
from actionlog import *


if __name__ == '__main__':
    app.run(host='0.0.0.0', \
            port=8012, \
            debug=True, \
            threaded=True)
