from flask import Flask, render_template, flash, \
            request, session, \
            redirect, url_for, logging, send_file

from flask_wtf import FlaskForm
from wtforms import StringField, TextAreaField, BooleanField, PasswordField, SelectField
from wtforms import validators
from wtforms import widgets, Form as _Form
from wtforms.validators import InputRequired

from wtforms.fields.html5 import DateField
from wtforms.fields.html5 import DateTimeField

from wtforms.ext.sqlalchemy.fields import QuerySelectField

import pandas as pd
from io import BytesIO

import time
from datetime import datetime, timedelta

from flask_sqlalchemy import SQLAlchemy
from sqlalchemy.sql import text as SQLQuery

import datamodel

from flask_mail import Message
from main import app, db, mail, setting

from email_func import *

from flask_login import login_required

from models import WebLog


def chkNull(val):
    if val == None:
        return ''
    else:
        return val


def MakeChoice(whChoice=''):
    try:
        strSQL = ''
        if whChoice.upper() == 'DEPT':
            strSQL = "SELECT * FROM ( " + \
                     "  SELECT distinct(dept_id) as dept_id " + \
                     "  FROM department " + \
                     "  UNION " + \
                     "  SELECT distinct(department) as dept_id " + \
                     "  FROM employee " + \
                     ")tbl1 " + \
                     "ORDER BY dept_id "

        if strSQL != '':
            sql = SQLQuery(strSQL)
            choices = db.session.execute(sql).fetchall()
        else:
            choices = ''

        return choices
    except Exception as e:
        print('MakeChoice: ' + str(e))
    

class SmockForm(FlaskForm):
    stDate = DateField("Start Date", format="'%Y-%m-%d'")
    enDate = DateField("End Date", format='%Y-%m-%d')
    empId = StringField("Employee ID")
####    deptID = SelectField("Department",
####                              choices=MakeChoice())
##    deptID = QuerySelectField('Department',
##                              query_factory=lambda: MakeChoice('DEPT'),
##                              get_label='dept_id',
##                              allow_blank=True,
##                              blank_text='',
##                              get_pk=lambda x: x.dept_id)
    smockStatus = SelectField(u'Smock Status',
                                  choices=[
                                          ('', ''), \
                                          ('Pass', 'Pass'), \
                                          ('Fail', 'Fail'), \
                                          ('Fail or No Test Last Result', 'Fail or No Test Last Result'), \
                                          ('No Test Result ONLY', 'No Test Result ONLY') \
                                        ] \
                              )
    disformat = SelectField(u'Display Format', choices=[('Multiple Tables', 'Multiple Tables'),('Single Table', 'Single Table')])
    output = SelectField(u'Output Report To', choices=[('Web Browser', 'Web Browser'),('Excel', 'Excel')])


    def validate_on_submit(self):
        result = super(SmockForm, self).validate()
        if self.stDate.data is not None and self.enDate.data is not None:
            if (self.stDate.data > self.enDate.data):
                return False
            else:
                return True


@app.route('/smock/<sql>')
@app.route('/smock', methods=['GET', 'POST'])
@app.route('/smockreporting', methods=['GET', 'POST'])
@login_required
def smock(sql=None, menu='dea'):
    if __name__ != '__main__':
        if 'username' not in session:
            return redirect('/login')

        updlog = WebLog(timedate = datetime.now(),
                        action_taken = 'GET',
                        action_page = str(request.url).rsplit('/', 1)[1],
                        upd_by = session['userid'])
        db.session.add(updlog)
        db.session.commit()
        
    form = SmockForm(request.form)
    table = datamodel.SmockTesting()
    disformat = "MULTIPLE TABLES"
    return render_template('smockfilterform.html', form=form , table=table, mydisformat=disformat)




@app.route('/smockfilter', methods=['GET', 'POST'])
def smockfilter(sql=None, menu='dea'):
    if __name__ != '__main__':
##        if 'username' not in session:
##            return redirect('/login')

##        #Need add this otherwise enter address bar to refresh then error?????
##        ddata = ()


        updlog = WebLog(timedate = datetime.now(),
                        action_taken = 'GET',
                        action_page = str(request.url).rsplit('/', 1)[1],
                        upd_by = session['userid'])
        db.session.add(updlog)
        db.session.commit()

                
        table = datamodel.SmockTesting()
        reportmsg = {'header':[], 'body':[], 'footer':[]}
        cdata = ['Date/Time', 'BadgeNo', 'SmockID', 'EmpName', 'Dept', 'P/L', 'Result', 'Value', 'SMlo', 'SMHi']

        QueryString = request.args.lists()
        
        if request.method == 'POST' or QueryString > 0:
            stDate = request.args.get('stdate')
            enDate = request.args.get('endate')
            empId = request.args.get('empid')
            smockStatus = request.args.get('smockstatus')
            email = request.args.get('email')
            disformat = request.args.get('disformat')
            
            if email is None:
                email = 'FALSE'

            emailsubject = "ESD Smock Report {} to {}".format(stDate, enDate)

            if request.method == 'POST':
                if stDate is None:
                    stDate = request.form['stDate']
                if enDate is None:
                    enDate = request.form['enDate']
                if empId is None:
                    empId = request.form['empId']
                if smockStatus is None:
                    smockStatus = request.form['smockStatus']
                if disformat is None:
                    disformat = request.form['disformat']

            if disformat is None:
                disformat = "MULTIPLE TABLES"
                

            wherestDate = ''
            whereenDate = ''
            whereempId = ''
            wheresmockStatus = ''

            strStDate = stDate
            strEnDate = enDate

            #print(chkNull(stDate))
            
            if stDate is not None and stDate != '':
                wherestDate = " and timedate >= '" + str(stDate) + "' "
            if enDate is not None and enDate != '':
                tempStr = datetime.strptime(enDate, '%Y-%m-%d') + timedelta(days=1)
                tempStr1 = datetime.strftime(tempStr, '%Y-%m-%d')
                whereenDate = " and timedate <= '" + str(tempStr1) + "' "

            if wherestDate == '' and whereenDate != '':
                wherestDate = " and timedate >= '" + str(enDate) + "' "
            if wherestDate != '' and whereenDate == '':
                tempStr = datetime.strptime(stDate, '%Y-%m-%d') + timedelta(days=1)
                tempStr1 = datetime.strftime(tempStr, '%Y-%m-%d')
                whereenDate = " and timedate <= '" + str(tempStr1) + "' "
            
            if empId is not None and empId != '':
                whereempId = " and emp_id = '" + str(empId) + "' "
            if smockStatus is not None and smockStatus != '':
                if smockStatus.upper().find('NO TEST') >= 0:
                    pass
                else:
                    wheresmockStatus = " and result = '" + str(smockStatus) + "' "


            subSQL = "SELECT timedate, cardno, emp_id, emp_name, sup_id, " + \
                     "dept_id, sm, result, smval, smlo, smhi " + \
                     "FROM smockresult " + \
                     "WHERE 1=1 " + \
                     wherestDate + \
                     whereenDate + \
                     whereempId + \
                     wheresmockStatus

            if setting.dbtype.upper() != 'MSSQL':
                subSQL = subSQL + " ORDER by sup_id, emp_id, timedate "

            
            if stDate == '' or enDate == '':
                #print(setting.dbtype.upper())
                if setting.dbtype.upper() == 'MSSQL':
                    subSQL = "SELECT TOP 20000 timedate, cardno, emp_id, emp_name, sup_id, " + \
                             "dept_id, sm, result, smval, smlo, smhi " + \
                             "FROM smockresult " + \
                             "WHERE 1=1 " + \
                             wherestDate + \
                             whereenDate + \
                             whereempId + \
                             wheresmockStatus
                    subSQL = subSQL + " ORDER by sup_id, emp_id, timedate "
                else:
                    subSQL = subSQL + " limit 20000 "

            if smockStatus is not None and smockStatus != '':
                if smockStatus.upper().find('NO TEST') >= 0 \
                   and smockStatus.upper().find('FAIL') >= 0:
                    subSQL2 = subSQL
                    subSQL = "SELECT tbl1.* FROM ( " + \
                             subSQL + \
                             ") tbl1 " + \
                             "WHERE tbl1.timedate = ( " + \
                             "                      SELECT MAX(tbl2.timedate) " + \
                             "                      FROM smockresult tbl2 " + \
                             "                      WHERE tbl2.emp_id=tbl1.emp_id) " + \
                             "AND UPPER(result) = 'FAIL' " + \
                             "UNION " + \
                             "SELECT '', card_no, emp_id, emp_name, sup_id, department, '', 'NoTest', '', '', '' " + \
                             "FROM employee " + \
                             "WHERE emp_id NOT IN (SELECT emp_id from ( " + \
                             subSQL + ")tblEmp " + \
                             ") AND emp_id <> 'DEFAULT' "

                elif smockStatus.upper().find('NO TEST') >= 0 \
                   and smockStatus.upper().find('ONLY') >= 0:
                    subSQL2 = subSQL
                    subSQL = "SELECT '' AS timedate, card_no AS cardno, emp_id, " + \
                             "emp_name, sup_id, department AS dept_id, '' AS sm, " + \
                             " 'NoTest' AS result, '' AS smval, " + \
                             " '' AS smlo, '' AS smhi " + \
                             "FROM employee " + \
                             "WHERE emp_id NOT IN (SELECT emp_id from ( " + \
                             subSQL + ")tblEmp " + \
                             ") AND emp_id <> 'DEFAULT' "
                
            
            subSQL = "SELECT timedate, cardno, emp_id, emp_name, " + \
                     "dept_id, sm, result, smval, smlo, smhi, " + \
                     "sup_id, supname, supemail " + \
                     "FROM (" + subSQL + ")tbl1 " + \
                     "LEFT JOIN ( " + \
                     "SELECT emp_id AS empid, emp_name AS supname, email AS supemail " + \
                     "FROM employee)tbl2 " + \
                     "ON tbl2.empid=tbl1.sup_id "

            subSQL = subSQL + " ORDER by sup_id, emp_id, timedate DESC "
            
            #flash('SQL: ' + subSQL)
            #reportmsg['body'].append('<b>SQL:</b> ' + subSQL + '<br>')
            sql = SQLQuery(subSQL)
            ddata = db.session.execute(sql).fetchall()
            
            table = datamodel.SmockTesting()
            #print(ddata)

            reportmsg['header'].append('Start Date is : <b>{}</b> ' \
                                       '&emsp;' \
                                       'End Date is : <b>{}</b> '.format(stDate, enDate))

            if ddata == None or ddata == () or ddata == []:
                #flash('No Record Found for this Criteria', 'info')
                reportmsg['body'].append('No Record Found for this Criteria')
                ddata = ()
                return render_template('smockfilter.html', data=ddata, table=table, mycolumns=cdata, mydisformat=disformat, reportmsg=reportmsg)
            elif request.method=='POST' and request.form['output'].upper() =='EXCEL':
                #ddata = tuple([cdata]) + tuple(ddata)

                cdata.append('SupID')
                cdata.append('SupName')
                cdata.append('SupEmail')

                header = cdata

##                if disformat.upper().find('MULTIPLE') >= 0:
##                    header = list(zip(['HEADER'] * len(cdata), header))
##                    header = pd.MultiIndex.from_tuples(header, names=['', ''])
                
                df = pd.DataFrame(ddata, columns=header)
                output = BytesIO()
                writer = pd.ExcelWriter(output, engine='xlsxwriter')

                df.reset_index(drop=True, inplace=True)
                df.index = df.index + 1

                xlheader = pd.DataFrame(df.columns.tolist()).T
                xlheader.to_excel(writer, header=False, index=False)
                df.columns = pd.Index(range(len(df.columns)))
                df.to_excel(writer, header=False, index=False, startrow=len(xlheader))

                #df.to_excel(writer)
                workbook  = writer.book
                writer.close()
                
                output.seek(0)


                updlog = WebLog(timedate = datetime.now(),
                                action_taken = 'EXPORT',
                                action_page = str(request.url).rsplit('/', 1)[1],
                                upd_by = session['userid'],
                                action_sql = subSQL)
                db.session.add(updlog)
                db.session.commit()
                
            
                return send_file(output, \
                                 attachment_filename="SmockReport.xlsx", \
                                 as_attachment=True)
            
            elif email.upper() == 'TRUE':
                if disformat.upper().find('MULTIPLE') >= 0:
                    lastsupid = "yyy"
                    currsupid = "yyy"
                    curremail = ""
                    lastemail = ""

                    mytable = ()
                    htmlheader = ""
                    
                    for row in ddata:
                        currsupid = row[10]
                        curremail = row[12]

                        currsupname = row[11]

##                        if currsupname is not None:
##                            htmlheader = "Dear " + currsupname + ", " + \
##                                         setting.emailheader

                        if lastsupid != currsupid:
                            if mytable != ():
                                if lastemail is not None:
                                    htmlbody = render_template('smockfilter.html', data=mytable, table=table, mycolumns=cdata, mydisformat=disformat)
                                    htmlbody = htmlheader + htmlbody
                                    send_email(subject = emailsubject, \
                                               recipients = lastemail, \
                                               html_body=htmlbody)
                                    print("Sent")
                                else:
                                    print("No Sent")

                                mytable = ()
                            
                        
                        lastsupid = currsupid
                        lastemail = curremail

                        mytable = mytable + (row,)
                    

                    if lastemail is not None:
                        htmlbody = render_template('smockfilter.html', data=mytable, table=table, mycolumns=cdata, mydisformat=disformat)
                        htmlbody = htmlheader + htmlbody
                        send_email(subject = emailsubject, \
                                   recipients = lastemail, \
                                   html_body=htmlbody)
                else:
                    htmlbody = render_template('smockfilter.html', data=ddata, table=table, mycolumns=cdata, mydisformat=disformat)
                    htmlbody = setting.emailintro + htmlbody
                    send_email(subject = emailsubject, \
                               recipients = setting.esdsup, \
                               text_body = setting.emailintro, \
                               html_body=htmlbody)
                        
            else:
                if (stDate is not None and stDate != '') \
                   and (enDate is not None and enDate != ''):
                    #flash('Start Date is : {} End Date is : {}'.format(stDate, enDate), 'info')
                    pass
                else:
                    if (stDate is not None and stDate != '') \
                       and (enDate is None or enDate == ''):
                        #flash('Start Date is : {} End Date is : {}'.format(stDate, stDate), 'info')
                        pass
                    elif (stDate is None or stDate == '') \
                         and (enDate is not None and enDate != ''):
                        #flash('Start Date is : {} End Date is : {}'.format(enDate, enDate), 'info')
                        pass
                    else:
                        #reportmsg['header'].append('No Selected Start Date or End Date report will only show maximum 20000 records')
                        flash('No Selected Start Date or End Date report will only show maximum 20000 records', 'info')
                    
            
    return render_template('smockfilter.html', data=ddata, table=table, mycolumns=cdata, mydisformat=disformat, reportmsg=reportmsg)





if __name__ == '__main__':
    print "this is a test in main 2"
    app.run(host='0.0.0.0', \
            port=4012, \
            debug=True, \
            threaded=True)
